* Create the database by copying it from the sample data.

close databases
if not MAKESAMP()
	return .F.
endif not MAKESAMP()

* Create a PRG with some stored procedure code.

text to lcProc noshow

procedure DBC_AfterOpenTable(cTableName)
local lcDir
lcDir = addbs(justpath(dbc()))
do case
	case used('AccessLog')
	case file(lcDir + 'AccessLog.dbf')
		use (lcDir + 'AccessLog') again in 0 shared
	otherwise
		create table (lcDir + 'AccessLog') free ;
			(Table C(20), ;
			When T)
endcase
insert into AccessLog values (cTableName, datetime())
use in AccessLog
set null off

function NextID(tcAlias, tcField)
local llUsed, ;
	lnCurrReprocess, ;
	liKey, ;
	lcMax, ;
	laID[1], ;
	luReturn

* Open the NEXTID table (if necessary).

llUsed = used('NEXTID')
if not llUsed
	use NEXTID again shared in 0
endif not llUsed

* Try to find a record for the field. If there isn't one, create it.

lnCurrReprocess = set('REPROCESS')
set reprocess to 10 seconds
if not seek(padr(upper(tcAlias), len(NEXTID.TABLE)), 'NEXTID', 'NEXTID')
	if empty(tcField) or type(tcAlias + '.' + tcField) = 'U'
		liKey = 0
	else
		lcMax = iif(type(tcAlias + '.' + tcField) = 'C', ;
			'val(' + tcField + ')', tcField)
		select max(&lcMax) from (tcAlias) into array laID
		liKey = iif(_tally > 0, laID[1], 0)
	endif empty(tcField) ...
	insert into NEXTID (TABLE, KEY) values (upper(tcAlias), liKey)
endif not seek(padr(tcAlias, ...

* If we can lock the record, increment the KEY value (rolling over to 1 if
* necessary) and assign it to luReturn.

if rlock('NEXTID')
	liKey = iif(NEXTID.KEY >= 2^32/2 - 1, 1, NEXTID.KEY + 1)
	replace KEY with liKey in NEXTID
	luReturn = iif(type(tcAlias + '.' + tcField) = 'C', ltrim(str(liKey)), ;
		liKey)
	unlock in NEXTID
endif rlock('NEXTID')

* Cleanup and return.

if not llUsed
	use in NEXTID
endif not llUsed
set reprocess to lnCurrReprocess
return luReturn

endtext
strtofile(lcProc, 'TEMP.PRG')

* Open the database, define a DefaultValue for ORDERS.ORDER_ID, and create a
* NEXTID table.

open database data\testdata
alter table Orders alter column Order_ID C(6) ;
	default NextID('Orders', 'Order_ID')
if file('Data\NextID.DBF')
	erase Data\NextID.DBF
endif file('Data\NextID.DBF')
if file('Data\AccessLog.DBF')
	erase Data\AccessLog.DBF
endif file('Data\AccessLog.DBF')
create table Data\NextID (Table C(10), Key I)
index on upper(Table) tag NextID

* Append the PRG to the stored procedures, nuke the PRG, and turn on DBC
* events.

append procedures from temp.prg overwrite
erase temp.prg
dbsetprop('testdata', 'database', 'dbcevents', .t.)

* Do something with the DBC natively.

insert into Orders (Order_Date, To_Name) values (date(), 'Native access')

* Now close the DBC and do something through the OLE DB provider.

close databases
local loConn as ADODB.Connection, ;
	loRS as ADODB.RecordSet, ;
	loCommand as ADODB.Command
loConn    = createobject('ADODB.Connection')
loRS      = createobject('ADODB.RecordSet')
loCommand = createobject('ADODB.Command')
loConn.ConnectionString = 'Provider=vfpoledb;Data Source=' + ;
	fullpath('Data\Testdata.DBC')
loConn.Open()
loRS.ActiveConnection = loConn
loRS.CursorLocation   = 3  && adUseClient
loRS.CursorType       = 3  && adOpenStatic
loRS.LockType         = 3  && adLockOptimistic
loRS.Open('select * from Orders')
wait window transform(loRS.RecordCount) + ' records were returned'

loCommand.ActiveConnection = loConn
loCommand.CommandText = 'insert into Orders (Order_Date, To_Name) ' + ;
	"values (date(), 'From OLE DB')"
loCommand.Execute()

loRS.Close()
loConn.Close()

* Now see the changes.

use Data\AccessLog
browse
open database Data\Testdata
use Orders
go bottom
browse
